==> kite-1.0b6.tar.gz: kite-1.0b6/vm/kite_stack.c <==
/*****************************************************************************
 * Copyright (c) 2007, Mooneer Salem
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Kite Language organization nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MOONEER SALEM ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL MOONEER SALEM BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#include <stdlib.h>
#include <assert.h>
#include "objs/kite_object.h"
#include "kite_vm.h"

/*****************************************************************************
 * Push an object onto the stack
 ****************************************************************************/
void kite_push_stack(kite_stack_t **st, int ref, void *obj)
{
    assert(st != NULL && obj != NULL);

    kite_stack_t *cur = *st;
    if (cur == NULL)
    {
        cur = malloc(sizeof(kite_stack_t));
        assert(cur != NULL);
        cur->length = 0;
        cur->allocated = sizeof(kite_stack_entry_t);
        cur->stack = malloc(cur->allocated);
        *st = cur;
    }

    if (((cur->length + 1) * sizeof(kite_stack_entry_t)) > cur->allocated)
    {
        int newsize = cur->allocated << 1;
        cur->stack = realloc(cur->stack, newsize);
        assert(cur->stack != NULL);
        cur->allocated = newsize;
    }

    kite_stack_entry_t *entry = &cur->stack[++cur->length - 1];
    entry->obj = obj;
    entry->reference = ref;
}

/*****************************************************************************
 * Pop an object from the stack
 ****************************************************************************/
void *kite_pop_stack(kite_stack_t **st)
{
    kite_stack_t *entry;
    void *ret;
    
    assert(st != NULL && (*st)->length > 0);
    entry = *st;

    ret = kite_top_stack(entry);
    entry->length--;

    return ret;
}

/*****************************************************************************
 * View top of stack
 ****************************************************************************/
#ifndef INLINE_STACK
void *kite_top_stack(kite_stack_t *st)
{
    kite_stack_entry_t *entry;

    if (!st || st->length == 0) return NULL;

    entry = &st->stack[st->length - 1];
    if (entry->reference) {
        return ((kite_symtab_t*)entry->obj)->value;
    }
    return entry->obj;
}
#endif /* INLINE_STACK */

/*****************************************************************************
 * Destroy stack
 ****************************************************************************/
void kite_destroy_stack(kite_stack_t **st)
{
    if (!*st) return;

    free((*st)->stack);
    free(*st);
    *st = NULL;
}

==> kite-1.0.0.tar.gz: kite-1.0.0/vm/kite_stack.c <==
/*****************************************************************************
 * Copyright (c) 2007, Mooneer Salem
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *     * Redistributions of source code must retain the above copyright
 *       notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above copyright
 *       notice, this list of conditions and the following disclaimer in the
 *       documentation and/or other materials provided with the distribution.
 *     * Neither the name of the Kite Language organization nor the
 *       names of its contributors may be used to endorse or promote products
 *       derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY MOONEER SALEM ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL MOONEER SALEM BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ****************************************************************************/

#include <stdlib.h>
#include <assert.h>
#include "objs/kite_object.h"
#include "kite_vm.h"

/*****************************************************************************
 * Push an object onto the stack
 ****************************************************************************/
void kite_push_stack(kite_stack_t **st, int ref, void *obj)
{
    kite_stack_t *cur;
    kite_stack_entry_t *entry;

    assert(st != NULL && obj != NULL);
    cur = *st;

    if (cur == NULL)
    {
#ifndef HAVE_GC_H
        cur = (kite_stack_t*)malloc(sizeof(kite_stack_t));
#else
        cur = (kite_stack_t*)GC_malloc(sizeof(kite_stack_t));
#endif /* HAVE_GC_H */
        assert(cur != NULL);
        cur->length = 0;
        cur->allocated = sizeof(kite_stack_entry_t);
#ifndef HAVE_GC_H
        cur->stack = (kite_stack_entry_t*)malloc(cur->allocated);
#else
        cur->stack = (kite_stack_entry_t*)GC_malloc(cur->allocated);
#endif /* HAVE_GC_H */
        assert(cur->stack != NULL);
        *st = cur;
    }

    if (((cur->length + 1) * sizeof(kite_stack_entry_t)) > cur->allocated)
    {
        int newsize = (cur->length << 1) * sizeof(kite_stack_entry_t);
#ifndef HAVE_GC_H
        cur->stack = (kite_stack_entry_t*)realloc(cur->stack, newsize);
#else
        cur->stack = (kite_stack_entry_t*)GC_realloc(cur->stack, newsize);
#endif /* HAVE_GC_H */
        assert(cur->stack != NULL);
        cur->allocated = newsize;
    }

    entry = &cur->stack[++cur->length - 1];
    entry->obj = obj;
    entry->reference = ref;
}

/*****************************************************************************
 * Pop an object from the stack
 ****************************************************************************/
void *kite_pop_stack(kite_stack_t **st)
{
    kite_stack_t *entry;
    void *ret;
    
    assert(st != NULL && (*st)->length > 0);
    entry = *st;

    ret = kite_top_stack(entry);
    entry->length--;

    return ret;
}

/*****************************************************************************
 * View top of stack
 ****************************************************************************/
#ifndef INLINE_STACK
void *kite_top_stack(kite_stack_t *st)
{
    kite_stack_entry_t *entry;

    if (!st || st->length == 0) return NULL;

    entry = &st->stack[st->length - 1];
    if (entry->reference) {
        return ((kite_symtab_t*)entry->obj)->value;
    }
    return entry->obj;
}
#endif /* INLINE_STACK */

/*****************************************************************************
 * Destroy stack
 ****************************************************************************/
void kite_destroy_stack(kite_stack_t **st)
{
    if (!*st) return;

#ifndef HAVE_GC_H
    free((*st)->stack);
    free(*st);
#endif /* HAVE_GC_H */
    *st = NULL;
}
